
{*******************************************************}
{                                                       }
{       Borland Delphi Runtime Library                  }
{       DB aware Web Browser                            }
{                                                       }
{       Copyright (C) 2001 Vino Rodrigues               }
{       vinorodrigues@yahoo.com                         }
{                                                       }
{       Version History:                                }
{       ----------------                                }
{                                                       }
{       1.1  30-Aug-01                                  }
{            First time I included "version history".   }
{                                                       }
{*******************************************************}

unit DBWebBrw;

{$UNDEF DEBUG}
{.$DEFINE DEBUG}

{$IFDEF DEBUG}
  {$D+} // DEBUG
  {$C+}	// ASSERTIONS
  {$O-}	// OPTIMIZATION
  {$Q+}	// OVERFLOWCHECKS
  {$R+}	// RANGECHECKS
  {$Y+} // SYMBOL AND X-REF INFO
  {$L+} // LOCAL SYMBOL INFO
{$ELSE}
  {$D-} // DEBUG
  {$C-}	// ASSERTIONS
  {$Q-} // OVERFLOWCHECKS
  {$R-} // RANGECHECKS
  {$Y-} // SYMBOL AND X-REF INFO
  {$L-} // LOCAL SYMBOL INFO
{$ENDIF}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  {$IFDEF VER140} Variants, {$ENDIF}
  OleCtrls, OleServer, SHDocVw, MSHTML, ActiveX, Db, DBCtrls;

type
  TDBWebBrowser = class(TWebBrowser)
  private
    FDataLink: TFieldDataLink;
    FOnDocumentComplete: TWebBrowserDocumentComplete;
    procedure DataChange(Sender: TObject);
    procedure DocumentComplete(Sender: TObject; const pDisp: IDispatch;
      var URL: OleVariant);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
  protected
    procedure LoadDocument; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    {$WARNINGS OFF}
    property OnDocumentComplete: TWebBrowserDocumentComplete
      read FOnDocumentComplete write FOnDocumentComplete;
    {$WARNINGS ON}
  end;

implementation

resourcestring
  sLoadingPrompt = '<I><B>Loading . . .</B></I>';
  sEmptyPrompt = '<I>(Empty)</I>';
  sNilPrompt = '<I><B>(Nil)</B></I>';

{ TDBWebBrowser }

constructor TDBWebBrowser.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataLink := TFieldDataLink.Create;
  FDataLink.OnDataChange := DataChange;
  inherited OnDocumentComplete := DocumentComplete;
end;

procedure TDBWebBrowser.DataChange(Sender: TObject);
begin
  Navigate('about:' + sLoadingPrompt);
end;

destructor TDBWebBrowser.Destroy;
begin
  inherited OnDocumentComplete := nil;
  FDataLink.OnDataChange := nil;
  FDataLink.Free;
  inherited Destroy;
end;

procedure TDBWebBrowser.DocumentComplete(Sender: TObject;
  const pDisp: IDispatch; var URL: OleVariant);
begin
  LoadDocument;
  if Assigned(FOnDocumentComplete) then
    FOnDocumentComplete(Sender, pDisp, URL);
end;

procedure TDBWebBrowser.LoadDocument;
var
  v: Variant;
  HTMLDocument: IHTMLDocument2;
begin
  HTMLDocument := Document as IHTMLDocument2;
  if Assigned(HTMLDocument) then
  begin
    v := VarArrayCreate([0, 0], varVariant);
    if FDataLink.Field = nil then
      v[0] := sNilPrompt
    else
      if FDataLink.Field.AsString = '' then
        V[0] := sEmptyPrompt
      else
        v[0] := FDataLink.Field.AsString;
    HTMLDocument.Write(PSafeArray(TVarData(v).VArray));
    HTMLDocument.Close;
  end;
end;

function TDBWebBrowser.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

function TDBWebBrowser.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TDBWebBrowser.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

procedure TDBWebBrowser.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

end.
